@@ -31,6 +31,7 @@ import hudson.Util;
 import hudson.FilePath;
 import hudson.console.AnnotatedLargeText;
 import hudson.console.ExpandableDetailsNote;
+import hudson.model.listeners.RunListener;
 import hudson.slaves.WorkspaceList;
 import hudson.slaves.NodeProperty;
 import hudson.slaves.WorkspaceList.Lease;
@@ -56,6 +57,7 @@ import hudson.util.AdaptedIterator;
 import hudson.util.Iterators;
 import hudson.util.LogTaskListener;
 import hudson.util.VariableResolver;
+import jenkins.model.Jenkins;
 import org.kohsuke.stapler.Stapler;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
@@ -169,9 +171,9 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
      */
     public Node getBuiltOn() {
         if (builtOn==null || builtOn.equals(""))
-            return Hudson.getInstance();
+            return Jenkins.getInstance();
         else
-            return Hudson.getInstance().getNode(builtOn);
+            return Jenkins.getInstance().getNode(builtOn);
     }
 
     /**
@@ -183,6 +185,24 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
         return builtOn;
     }
 
+    /**
+     * Gets the nearest ancestor {@link AbstractBuild} that belongs to
+     * {@linkplain AbstractProject#getRootProject() the root project of getProject()} that
+     * dominates/governs/encompasses this build.
+     *
+     * <p>
+     * Some projects (such as matrix projects, Maven projects, or promotion processes) form a tree of jobs,
+     * and still in some of them, builds of child projects are related/tied to that of the parent project.
+     * In such a case, this method returns the governing build.
+     *
+     * @return never null. In the worst case the build dominates itself.
+     * @since 1.421
+     * @see AbstractProject#getRootProject()
+     */
+    public AbstractBuild<?,?> getRootBuild() {
+        return this;
+    }
+
     /**
      * Used to render the side panel "Back to project" link.
      *
@@ -393,6 +413,11 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
          *      Passed in for the convenience. The returned path must be registered to this object.
          */
         protected Lease decideWorkspace(Node n, WorkspaceList wsl) throws InterruptedException, IOException {
+            String customWorkspace = getProject().getCustomWorkspace();
+            if (customWorkspace != null) {
+                // we allow custom workspaces to be concurrently used between jobs.
+                return Lease.createDummyLease(n.getRootPath().child(getEnvironment(listener).expand(customWorkspace)));
+            }
             // TODO: this cast is indicative of abstraction problem
             return wsl.allocate(n.getWorkspaceFor((TopLevelItem)getProject()));
         }
@@ -401,12 +426,12 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
             Node node = getCurrentNode();
             assert builtOn==null;
             builtOn = node.getNodeName();
-            hudsonVersion = Hudson.VERSION;
+            hudsonVersion = Jenkins.VERSION;
             this.listener = listener;
 
             launcher = createLauncher(listener);
-            if (!Hudson.getInstance().getNodes().isEmpty())
-                listener.getLogger().println(node instanceof Hudson ? Messages.AbstractBuild_BuildingOnMaster() : Messages.AbstractBuild_BuildingRemotely(builtOn));
+            if (!Jenkins.getInstance().getNodes().isEmpty())
+                listener.getLogger().println(node instanceof Jenkins ? Messages.AbstractBuild_BuildingOnMaster() : Messages.AbstractBuild_BuildingRemotely(builtOn));
 
             final Lease lease = decideWorkspace(node,Computer.currentComputer().getWorkspaceList());
 
@@ -475,7 +500,14 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
 
             buildEnvironments = new ArrayList<Environment>();
 
-            for (NodeProperty nodeProperty: Hudson.getInstance().getGlobalNodeProperties()) {
+            for (RunListener rl: RunListener.all()) {
+                Environment environment = rl.setUpEnvironment(AbstractBuild.this, l, listener);
+                if (environment != null) {
+                    buildEnvironments.add(environment);
+                }
+            }
+
+            for (NodeProperty nodeProperty: Jenkins.getInstance().getGlobalNodeProperties()) {
                 Environment environment = nodeProperty.setUp(AbstractBuild.this, l, listener);
                 if (environment != null) {
                     buildEnvironments.add(environment);
@@ -527,7 +559,7 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
                             AbstractBuild.this.scm = scm.createChangeLogParser();
                             AbstractBuild.this.changeSet = AbstractBuild.this.calcChangeSet();
 
-                            for (SCMListener l : Hudson.getInstance().getSCMListeners())
+                            for (SCMListener l : Jenkins.getInstance().getSCMListeners())
                                 l.onChangeLogParsed(AbstractBuild.this,listener,changeSet);
                             return;
                         }
@@ -646,7 +678,26 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
             } catch (AbstractMethodError e) {
                 mon = BuildStepMonitor.BUILD;
             }
-            return mon.perform(bs, AbstractBuild.this, launcher, listener);
+            Result oldResult = AbstractBuild.this.getResult();
+            boolean canContinue = mon.perform(bs, AbstractBuild.this, launcher, listener);
+            Result newResult = AbstractBuild.this.getResult();
+            if (newResult != oldResult) {
+                String buildStepName = getBuildStepName(bs);
+                listener.getLogger().format("Build step '%s' changed build result to %s%n", buildStepName, newResult);
+            }
+            if (!canContinue) {
+                String buildStepName = getBuildStepName(bs);
+                listener.getLogger().format("Build step '%s' marked build as failure%n", buildStepName);
+            }
+            return canContinue;
+        }
+
+        private String getBuildStepName(BuildStep bs) {
+            if (bs instanceof Describable<?>) {
+                return ((Describable<?>) bs).getDescriptor().getDisplayName();
+            } else {
+                return bs.getClass().getSimpleName();
+            }
         }
 
         protected final boolean preBuild(BuildListener listener,Map<?,? extends BuildStep> steps) {
@@ -677,7 +728,7 @@ public abstract class AbstractBuild<P extends AbstractProject<P,R>,R extends Abs
         if (scm==null) {
             // for historical reason, null means CVS.
             try {
-                Class<?> c = Hudson.getInstance().getPluginManager().uberClassLoader.loadClass("hudson.scm.CVSChangeLogParser");
+                Class<?> c = Jenkins.getInstance().getPluginManager().uberClassLoader.loadClass("hudson.scm.CVSChangeLogParser");
                 scm = (ChangeLogParser)c.newInstance();
             } catch (ClassNotFoundException e) {
                 // if CVS isn't available, fall back to something non-null.
